unit registration;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Registry, ComObj;

type EProgramID = class(Exception);

type
 TRegistration = class(TComponent)
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  function Status(PID: TGUID; Days: integer; Flag: boolean): Boolean;
  function Verify(PID: TGUID; RGID: string):Boolean;
 private
  FDays: integer;
  FLicense: string;
  FNumber: string;
  FExpired: Boolean;
  FRegistered: Boolean;
  procedure Compute;
  function GetLicense: string;
  procedure SetLicense(value: string);
 protected
  // protected
 published
  property License: string read GetLicense write SetLicense;
  property Number: string read FNumber write FNumber;
  property Days: Integer read FDays write FDays;
  property Expired: Boolean read FExpired write FExpired;
  property Registered: Boolean read FRegistered write FRegistered;
 end;

procedure Register;

implementation

//mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
procedure Register;
begin
 RegisterComponents('Donth',[TRegistration]);
end;

//mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
constructor TRegistration.Create(AOwner: TComponent);
var
 MyReg: TRegistry;
 temp: string;
begin
 inherited Create(AOwner);
 FDays := 0;
 MyReg := TRegistry.Create;
 MyReg.RootKey := HKEY_LOCAL_MACHINE;
 MyReg.OpenKey('\Software\Microsoft\Windows\CurrentVersion', TRUE);
 temp := MyReg.ReadString('ProductID');
 temp := copy(temp,11,7)+copy(temp,1,5);
 FLicense := copy(temp,1,4)+'-'+copy(temp,5,4) + '-' + copy(temp, 9, 4);
 Compute;
end;

//mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
destructor TRegistration.Destroy;
begin
 inherited Destroy;
end;

//mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
function TRegistration.GetLicense;
begin
 Result := FLicense;
end;

//mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
procedure TRegistration.SetLicense(value: string);
begin
 if value <> FLicense then begin
  Flicense := value;
  Compute;
 end;
end;

//mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
procedure TRegistration.Compute;
const mkr = '1703730891735894587126710684391856791750134568720576153982937652';
var
 rslt, dx: integer;
begin
 rslt := 0;
 FNumber := '';
 for dx := 1 to length(FLicense) do begin
  if FLicense[dx] = '-' then begin
   FNumber := FNumber + '-';
  end else begin
   rslt := rslt + ord(FLicense[dx]);
   FNumber := FNumber + mkr[(rslt mod 64)+1];
  end;
 end;
end;

//mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
function TRegistration.Verify(PID: TGUID; RGID: string):Boolean;
var
 Reg: TRegistry;
 guid: string;
begin
  // Make sure we have a valid GUID
 try
  Guid := GUIDToString(PID);
 except
  raise EProgramID.Create('Missing or invalid Program ID');
  exit;
 end;
 if RGID = FNumber then begin
  Reg := TRegistry.Create;
  Reg.RootKey := HKEY_LOCAL_MACHINE;
  Reg.OpenKey('\Software\CLASSES\CLSID\' + Guid + '\ProgID', TRUE);
  reg.WriteString('','FFFFFF');
  Reg.CloseKey;
  Reg.free;
  Result := TRUE;
  FRegistered := TRUE;
  FExpired := FALSE;
 end else begin
  Result := FALSE;
 end;
end;

//mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
function TRegistration.Status(PID: TGUID; Days: integer; Flag:boolean): Boolean;
var
 guid, temp: string;
 Reg: TRegistry;
 tmp, today: integer;
begin
 today := trunc(Date);     // Get today's date
  // Make sure we have a valid GUID
 try
  Guid := GUIDToString(PID);
 except
  raise EProgramID.Create('Missing or invalid Program ID');
  exit;
 end;
  // Get the Registry key
 Reg := TRegistry.Create;
 Reg.RootKey := HKEY_LOCAL_MACHINE;
 Reg.OpenKey('\Software\CLASSES\CLSID\' + Guid + '\ProgID', TRUE);
 temp := Reg.ReadString(''); // If a blank, we do not have a key

 if Flag = TRUE then temp := ''; // We want a NEW Registration record

 if temp = '' then begin // Create a new record
  if days = 0 then temp := 'FFFFFF' else temp := IntToHex(Days+today,6);
  Reg.WriteString('',temp);
 end;
 Reg.CloseKey;
 Reg.free;
 if temp = 'FFFFFF' then begin
  FDays := -1;
 end else begin
  FDays := StrToInt('$'+temp) - today;
  if FDays < 0 then FDays := 0;
 end;
 if FDays = -1 then begin // We are registered
  FRegistered := TRUE;
  FExpired := FALSE;
 end else if FDays = 0 then begin // We are expired
  FRegistered := FALSE;
  FExpired := TRUE;
 end else begin
  FRegistered := FALSE;
  FExpired := FALSE;
 end;
 Result := FExpired;
end;

end.
